<?php
/**
 * $Horde: chora/co.php,v 1.9.2.15 2003/01/03 12:48:49 jan Exp $
 *
 * Copyright 2000-2003 Anil Madhavapeddy <anil@recoil.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

require_once 'lib/base.php';
require_once 'horde-libs/mime_mapping.php';
require_once 'horde-libs/mime_drivers.php';
require_once 'horde-libs/MIME/Part.php';
require_once 'horde-libs/MIME/Magic.php';
require_once 'horde-libs/MIME/Viewer.php';

hasAccess($p_user->getUserId(), "/cvs/", "r", $group_id);

/* Should we pretty-print this output or not? */
$plain = $p;

/* Create the CVSLib_File object and populate it. */
$file = &CVSLib_File::getFileObject($CVS, $fullname, CVSLIB_LOG_FULL);

/* If no revision is specified, default to HEAD.
 * If a revision is specified, it's safe to cache for a long time */
if (!$r) {
    $r = $file->queryRevision();
    header('Cache-Control: max-age=60, must-revalidate');
} else {
    header('Cache-Control: max-age=2419200');
}

/* Is this a valid revision being requested? */
if (!CVSLib_rev::valid($r)) {
    fatal('404 Not Found', "Revision Not Found: $r is not a valid RCS revision number");
}

/* Retrieve the actual checkout. */
$checkOut = CVSLib_Checkout::get($CVS, $fullname, $r);

/* Calculate the file extension, to identify the MIME type. */
$extension = $file->getExtension();

/* Check error status, and either show error page, or the checkout contents */
if (is_object($checkOut) && $checkOut->id() == CVSLIB_ERROR) {
    checkError($checkOut);
} else if (!$plain) {
    /* Pretty-print the checked out copy */
    $pretty = CVSLib_Checkout::pretty($CVS, $extension, $checkOut);
    $log = $file->logs[$r];

    /* Get this revision's attributes in printable form. */
    $author = Chora::showAuthorName($log->queryAuthor(), true);
    $date = strftime('%c', $log->queryDate());
    $log_print = Chora::toHTML($log->queryLog());

    if ($pretty->getType() == 'text/html' || $pretty->getType() == 'text/plain') {
	chora_header(array('title' => "CVS Checkout of " . basename($fullname) . " (revision $r)", 'rev' => $r, 'subtab' => 'view'));

	echo "<p>$author ($date)</p>\n";
	echo "<p>$log_print</p>\n";
	echo "<hr size=\"1\" noshade=\"noshade\">\n";
        if ($pretty->getType() == 'text/plain') {
            echo '<pre>' . htmlspecialchars($pretty->render()) . '</pre>';
        } else {
            echo $pretty->render();
        }
	echo "<hr size=\"1\" noshade=\"noshade\">\n";
	chora_footer();
    } else {
        header('Content-Type: ' . $pretty->getType());
        print $pretty->render();
    }
} else {
    /* Send Force a save file dialog. */
    $filename = $file->queryName();
    header('Content-Type: ' . MIME_Magic::extToMIME($extension));
    header('Content-Disposition: attachment; filename=' . $filename);
    $fp = CVSLib_Checkout::get($CVS, $fullname, $r);
    $content = '';
    while ($line = fgets($fp, 4096)) {
        $content .= $line;
    }
    @fclose($fp);
    echo $content;
    exit;
}
?>
