<?php
/**
 * $Horde: chora/cvs.php,v 1.116.2.18 2003/01/03 12:48:48 jan Exp $
 *
 * Copyright 1999-2003 Anil Madhavapeddy <anil@recoil.org>
 * Copyright 1999-2003 Charles Hagenbuch <chuck@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * Changes made to fit UDAMan/SourceForge Copyright 2003, The MITRE Corporation
 *   Justin Richer <jricher@mitre.org>
 *
 */

require_once 'lib/base.php';

require_once 'horde-libs/mime_drivers.php';
require_once 'horde-libs/MIME/Magic.php';
require_once 'horde-libs/MIME/Viewer.php';

hasAccess($p_user->getUserId(), "/cvs/", "r", $group_id);

if (@is_dir($fullname)) {
    /* checkError() is the error trapping function. */
    checkError($dir = $CVS->queryDir($where));

    $atticFlags = $acts['sa'] ? CVSLIB_ATTIC_SHOW : CVSLIB_ATTIC_HIDE;

    checkError($dir->browseDir(CVSLIB_LOG_QUICK, $atticFlags));
    $dir->applySort($acts['sbt'], $acts['ord']);
    checkError($dirList =& $dir->queryDirList());
    checkError($fileList = $dir->queryFileList($atticFlags));

    /* Decide what title to display */
    if ($where == '') {
        $title = "Browse CVS Repository";
    } else {
        $title = "CVS Directory of /$where";
    }

    chora_header(array('title' => $title, 'subtab' => 'browse'));

    $atticSize = count($dir->atticFiles);
    if ($atticSize > 0) {

      if ($acts['sa']) {
        $extraLink='<input type="checkbox" checked="checked" onClick="document.location=\'' . Chora::url('cvs', $where, array('sa' => 0)) . "'\"> Show Deleted Files ($atticSize)";
      } else {
	$extraLink='<input type="checkbox" onClick="document.location=\'' . Chora::url('cvs', $where, array('sa' => 1)) . "'\"> Show Deleted Files ($atticSize)";
      }
      echo '<div align="left"><p>' . $extraLink . "</p></div>\n";
    }

    foreach (array('age', 'rev', 'name', 'author') as $u) {
        $umap = array('age' => CVSLIB_SORT_AGE, 'rev' => CVSLIB_SORT_REV,
                      'name' => CVSLIB_SORT_NAME, 'author' => CVSLIB_SORT_AUTHOR);
        $arg = array('sbt' => $umap[$u]);
        if ($acts['sbt'] == $umap[$u]) {
            $arg['ord'] = !$acts['ord'];
        }
        $url[$u] = Chora::url('cvs', $where, $arg);
    }

    /* Print out the directory header. */
    $printAllCols = sizeof($fileList);
?>
<?php
							;
    $dir_header = array();
    array_push($dir_header, array('title' => "File",
				  'link' => $url['name'],
				  'selected' => ($acts['sbt'] == CVSLIB_SORT_NAME)));
    array_push($dir_header, array('title' => "Rev",
				  'link' => $url['rev'],
				  'selected' => ($acts['sbt'] == CVSLIB_SORT_REV)));
    array_push($dir_header, array('title' => "Author",
				  'link' => $url['author'],
				  'selected' => ($acts['sbt'] == CVSLIB_SORT_AUTHOR)));
    array_push($dir_header, array('title' => "Date",
				  'link' => $url['age'],
				  'selected' => ($acts['sbt'] == CVSLIB_SORT_AGE)));
    array_push($dir_header, array('title' => "Last Log"));
    
    echo $HTML->build_list_table_top($dir_header);
       
    /* Unless we're at the top, display the 'back' bar. */
    if ($where != '') {
        $url = Chora::url('cvs', preg_replace('|[^/]+$|', '', $where));
	$dirrow = 0;
?>
<tr bgcolor="<?php echo html_get_alt_row_color($dirrow); ?>">
 <td>
   <a href="<?php echo $url ?>">
   <?php echo html_image('images/cvs/back.gif', 16, 16, array('alt' => "Back")) . '&nbsp; Parent Directory'; ?></a>
 </td>
 <td> &nbsp; </td>
 <td> &nbsp; </td>
 <td> &nbsp; </td>
 <td> &nbsp; </td>
</tr>
<?php
    }

    /* Display all the directories first. */
    foreach ($dirList as $currentDir) {
        if ($conf['hide_restricted'] && Chora::isRestricted($currentDir)) {
            continue;
        }
	$dirrow++;
        $url = Chora::url('cvs', "$where/$currentDir");
?>
<tr bgcolor="<?php echo html_get_alt_row_color($dirrow); ?>">
 <td>
<?php if (!Chora::isRestricted("$where/$currentDir")): ?>
   <a href="<?php echo $url ?>">
   <?php echo html_image('images/cvs/folder.gif', 16, 16, array('alt' => 'Directory')) ?>&nbsp;<?php echo $currentDir ?>/</a>
<?php else: ?>
   <?php echo html_image('images/cvs/folder.gif', 16, 16, array('alt' => 'Directory')) ?>&nbsp;<?php echo $currentDir ?>/
<?php endif; ?>
 </td>
 <td> &nbsp; </td>
 <td> &nbsp; </td>
 <td> &nbsp; </td>
 <td> &nbsp; </td>
</tr>
<?php
    }

    /* Display all of the files in this directory */
    foreach ($fileList as $currFile) {
        if ($conf['hide_restricted'] && Chora::isRestricted($currFile->queryName())) {
            continue;
        }
	$dirrow++;
        $lg = $currFile->queryLastLog();
        if ($lg == null) {
            continue;
        }
        $realname = $currFile->queryName();
        $mimeType = MIME_Magic::extToMIME(CVSLib_File::getExtension($realname));

        $icon = MIME_Viewer::getIcon($mimeType);

        $aid = $lg->queryAuthor();
        $author = Chora::showAuthorName($aid);
        $head = $currFile->queryHead();
        $date = $lg->queryDate();
        $log  = $lg->queryLog();
        $attic = $currFile->isAtticFile();
        $fileName = $where. ($attic ? '/Attic' : ''). "/$realname";
        $name = Text::htmlallspaces($realname);
        $url = Chora::url('cvs', $fileName);
        $readableDate = CVSLib_File::readableTime($date);
        if ($log) {
            $shortLog = str_replace("\n" , ' ',
                trim(substr($log, 0, $conf['options']['shortLogLength'] - 1)));
            if (strlen($log) > 80) {
                $shortLog .= "...";
            }
        }
	echo '<tr bgcolor="' . html_get_alt_row_color($dirrow) . "\">\n";
	if (!Chora::isRestricted($fileName)) {
	  echo "  <td nowrap=\"nowrap\">";
	  echo "<a href=\"$url\">";
	  if ($attic) {
	    echo html_image('images/cvs/deleted.gif', 16, 16, array('alt' => 'Deleted File')) . "&nbsp\n";
	  } else {
	    echo html_image($icon, 16, 16, array('alt' => 'File')) . "&nbsp\n";
	  }
	  echo "$name</a>\n";
	  echo " </td>\n";
	  echo " <td>\n";
	  echo '   &nbsp;<b><a href="' . Chora::url('co', $fileName, array('r' => $head)) . "\">$head</a></b>\n";
	  echo " </td>\n";
	} else {
	  echo " <td nowrap=\"nowrap\">\n";
	  if ($attic) {
	    echo html_image('images/cvs/deleted.gif', 16, 16, array('alt' => 'Deleted File')) . "&nbsp\n";
	  } else {
	    echo html_image($icon, 16, 16, array('alt' => 'File')) . "&nbsp\n";
	  }
	  echo $name;
?>
 </td>
 <td>
   &nbsp;<b><?php echo $head ?></b>
 </td>
<?php   } ?>
 <td>
   &nbsp;<?php 
    echo "$author\n";
?>
 </td>
 <td nowrap="nowrap">
   &nbsp;
   <i><?php echo $readableDate ?></i>
 </td>
 <td>
   &nbsp;
   <?php if (!empty($log)) : ?>
   <?php echo htmlspecialchars($shortLog) ?>
   <?php endif; ?>
 </td>
</tr>
<?php
    }
    /* Display the options control panel at the bottom */
    $formwhere = $scriptName . '/' . $where;

    echo "</table>\n";
    chora_footer();
} else if (@is_file($fullname . ',v')) {
    $fl = &CVSLib_File::getFileObject($CVS, $fullname, CVSLIB_LOG_FULL);

    $upwhere = preg_replace('|[^/]+$|', '', $where);

    $isBranch = isset($onb) && isset($fl->branches[$onb]) ? $fl->branches[$onb] : '';

    if ($isBranch) {
      $branch = $fl->branches[$onb];
    } else {
      $branch = '';
    }
    chora_header(array('title' => "CVS Log for $where", 'subtab' => 'log', 'branch' => $branch, 'onb' => $onb));

    echo '<table width="100%" cellspacing="0" cellpadding="6">' . "\n";
    $revRow = 0;
    foreach ($fl->logs as $lg) {
        $revRow++;
        $rev = $lg->rev;

        /* Are we sticking only to one branch ? */
        if ($onb && CVSLib_Rev::valid($onb)) {

            /* If so, if we are on the branch itself, let it through */
            if (substr($rev, 0, strlen($onb)) != $onb) {

                /* We are not on the branch, see if we are on a trunk
                 * branch below the branch */
                $baseRev = CVSLib_Rev::strip($onb, 1);

                /* Check we are at the same level of branching or less */
                if (substr_count($rev,'.') <= substr_count($baseRev,'.')) {
                    /* If we are at the same level, and the revision is
                     * less, then let the revision through, since it was
                     * committed before the branch actually took place
                     */
                    if (CVSLib_Rev::cmp($rev, $baseRev) > 0) {
                        continue;
                    }
                } else {
                    continue;
                }
            }
        }

        $textURL = Chora::url('co', $where, array('r' => $rev));
        $commitDate = strftime('%c', $lg->date);
        $readableDate = CVSLib_File::readableTime($lg->date, true);

        $aid = $lg->queryAuthor();
        $author = Chora::showAuthorName($aid, true);

        if (!empty($lg->tags)) {
            $commitTags = implode(', ', $lg->tags);
        } else {
            $commitTags = '';
        }

        $branchPointsArr = array();
        foreach ($lg->querySymbolicBranches() as $symb => $bra) {
            $branchPointsArr[] = '<a href="' . Chora::url('cvs', $where, array('onb' => $bra)) . '">'. $symb . '</a>';
        }

        /* Calculate the current branch name and revision */
        $branchPoints = implode(' , ', $branchPointsArr);
        $branchRev = CVSLib_Rev::strip($rev, 1);
        if (@isset($fl->branches[$branchRev])) {
            $branchName = $fl->branches[$branchRev];
        } else {
            $branchName = '';
        }

        if ($prevRevision = CVSLib_Rev::prev($lg->queryRevision())) {
            $changedLines = $lg->queryChangedLines();
            $diffURL = Chora::url('diff', $where, array('r1'=>$prevRevision,'r2'=>$rev, 'ty'=>'h'));
            $longDiffURL = Chora::url('diff', $where, array('r1'=>$prevRevision,'r2'=>$rev, 'ty'=>'h', 'num'=>10));
            $uniDiffURL = Chora::url('diff', $where, array('r1'=>$prevRevision,'r2'=>$rev,'ty'=>'u'));
            $nowsDiffURL = Chora::url('diff', $where, array('nows' => 1, 'r1'=>$prevRevision,'r2'=>$rev, 'ty'=>'h'));
            $nowsLongDiffURL = Chora::url('diff', $where, array('nows' => 1, 'r1'=>$prevRevision,'r2'=>$rev, 'ty'=>'h', 'num'=>10));
            $nowsUniDiffURL = Chora::url('diff', $where, array('nows' => 1, 'r1'=>$prevRevision,'r2'=>$rev,'ty'=>'u'));
        }

        $manyRevisions = !($fl->queryRevision() === '1.1');
        if ($manyRevisions) {
            $selCvsURL = Chora::url('cvs', $where, array('r1' => $rev, 'onb' => $onb));
            if (!empty($r1)) {
                $selDiffURL = Chora::url('diff', $where, array('r1'=>$r1, 'r2'=>$rev, 'ty'=>'h'));
                $selLongDiffURL = Chora::url('diff', $where, array('r1'=>$r1, 'r2'=>$rev, 'ty'=>'h', 'num'=>10));
                $selUniDiffURL = Chora::url('diff', $where, array('r1'=>$r1, 'r2'=>$rev,'ty'=>'u'));
            }
        }

        $logMessage = Chora::toHTML($lg->log);

        if ($r1 === $rev) {
	  $headerClass = "background: $HTML->LISTITEM_SELECTED_COLOR; color: $HTML->FONTCOLOR_CONTENT;";
        } else {
	  $headerClass = "background: $HTML->COLOR_HTMLBOX_TITLE; color: $HTML->FONTCOLOR_HTMLBOX_TITLE;";
        }

?>

<tr style="<?php echo $headerClass; ?>"><td align="left" style="<?php echo $headerClass; ?>">
<a name="rev<?php echo $rev ?>" />
<a style="<?php echo $headerClass; ?> text-decoration: underline;" href="<?php echo $textURL ?>"><?php echo $rev ?></a>
<?php 
   if (user_is_valid($aid)) {
     echo "by <a style=\"$headerClass; text-decoration: underline;\" href=\"/users/$aid\">$aid</a>\n";
   } else {
     echo "by $aid\n";
   }
?>
</td><td align="right" style="<?php echo $headerClass; ?>">
<a style="<?php echo $headerClass; ?> text-decoration: underline;" href="<?php echo Chora::url('annotate', $where, array('rev' => $rev)); ?>">Annotate</a> <b>|</b>
<a style="<?php echo $headerClass; ?> text-decoration: underline;" href="<?php echo Chora::url('co', $where, array('r' => $rev)); ?>">View</a> <b>|</b>
<a style="<?php echo $headerClass; ?> text-decoration: underline;" href="<?php echo Chora::url('co', $where, array('r' => $rev, 'p' => 1)); ?>">Download</a>
</td></tr>
<tr valign="top">
<td bgcolor="<?php echo $HTML->COLOR_LTBACK1; ?>" width="50%">
<?php
	echo "$commitDate <i>($readableDate ago)</i><br />\n";
	if (!empty($branchName) || !empty($commitTags) || !empty($branchPoints)) {
	  echo "<hr  noshade=\"noshade\" size=\"1\" width=\"95%\"/>\n";
	}
	if (!empty($branchName)) {
	  echo 'Branch: <b><a href="' . Chora::url('cvs', $where, array('onb' => $branchRev)) . "\">$branchName</a></b><br />\n";
	}
	if (!empty($commitTags)) {
	  echo "CVS Tags: <b>$commitTags</b><br/>\n";
	}
	if (!empty($branchPoints)) {
	  echo "Branch Point for: <b>$branchPoints</b><br />\n";
	}
	if (!empty($prevRevision) || $manyRevisions) {
	  echo "<hr noshade=\"noshade\" size=\"1\" width=\"95%\"/>\n";
	}
	if (!empty($prevRevision)) {
	  echo "Changed since <b>$prevRevision</b>: $changedLines\n";
	}
	if ($manyRevisions) {
	  if ($r1 === $rev) {
	    echo '<b>[<a href="' . Chora::url('cvs', $where) . "\">Unselect</a>]</b>\n";
	  } else {
	    echo "[<a href=\"$selCvsURL\">Select for Diff</a>]\n";
	  }
	}
	echo "<br />\n";
	if (!empty($prevRevision)) {
	  echo "<a href=\"$diffURL\">Diffs to version $prevRevision</a>\n";
	  echo "(<a href=\"$longDiffURL\">long</a>&nbsp;/&nbsp;";
	  echo "<a href=\"$uniDiffURL\">unified</a>)<br />\n";
	  echo "<a href=\"$nowsDiffURL\">Diffs ignoring whitespace to version $prevRevision</a>\n";
	  echo "(<a href=\"$nowsLongDiffURL\">long</a>&nbsp;/&nbsp;";
	  echo "<a href=\"$nowsUniDiffURL\">unified</a>)<br />\n";
	}
	if ($manyRevisions && !empty($r1) && ($r1 != $prevRevision) && ($r1 != $rev)) {
	  echo "<a href=\"$selDiffURL\">Diffs to selected version $r1</a>\n";
	  echo "(<a href=\"$selLongDiffURL\">long</a> /\n";
	  echo "<a href=\"$selUniDiffURL\">unified</a>)<br />\n";
	}
?>
<p> &nbsp; </p>
</td>
<td bgcolor="<?php echo $HTML->COLOR_LTBACK2; ?>" width="50%">
<?php echo $logMessage ?>
</td></tr>
<?php
    }
?>
</table>
<?php
    $first = end($fl->logs);
    $diffValueLeft  = $first->rev;
    $diffValueRight = $fl->queryRevision();

    $sel = '';
    foreach ($fl->symrev as $sm => $rv) {
        $sel .= '<option value="' . $rv . '">' . $sm . "</option>\n";
    }

    $selAllBranches = '';
    foreach ($fl->branches as $num => $sym) {
        $selAllBranches .= '<option value="' . $num . '"' . (($onb && $num == $onb) ? ' selected="selected"' : '') . '>' . $sym . "</option>\n";
    }

    //include CHORA_TEMPLATES . '/log/request.inc';
?>

<a name="diff"></a>
<br />
<table width="100%" cellspacing="0" cellpadding="4">
<tr><td>
 This form allows you to request diffs between any two revisions of a file.  You may select a symbolic revision name using the selection box or you may type in a numeric name using the type-in text box.
</td></tr>
<tr><td>
<form method="get" action="<?php echo Chora::url('diff',$where) ?>">
<?php echo Chora::generateHiddens() ?>
<input type="hidden" name="f" value="<?php echo $f; ?>" />
<table cellspacing="0" cellpadding="1" border="0">
<tr>
<td align="right">
Retrieve diffs between:
</td>
<td>
<select name="r1">
  <option value="0" selected="selected">Use Text Field</option>
  <?php echo $sel ?>
</select>
<input type="text" size="12" name="tr1" value="<?php echo $diffValueLeft ?>" />
</td>
<td>Type:</td>
<td>
<select name="ty">
 <option value="h" selected="selected">Human Readable</option>
 <option value="u">Unified</option>
 <option value="c">Context</option>
 <option value="s">Side-by-Side</option>
 <option value="e">Ed Script</option>
</select> &nbsp;
</td>
</tr>
<tr>
<td align="right">and:</td>
<td>
<select name="r2">
  <option value="0" selected="selected">Use Text Field</option>
  <?php echo $sel ?>
</select>
<input type="text" size="12" name="tr2" value="<?php echo $diffValueRight ?>" />
</td>
<td>&nbsp;</td>
<td><input type="submit" value="Get Diffs" /></td>
</tr>
</table>
</form>
</td></tr>
<?php 
if (!empty($selAllBranches)) {
?>
<tr><td>
<form method="get" action="<?php echo Chora::url('cvs', $where) ?>">
<?php echo Chora::generateHiddens() ?>
<table cellspacing="0" cellpadding="1" border="0">
<tr><td>View revisions on:</td>
<td>
<select name="onb">
  <option value="0" <?php if (!isset($onb) || !$onb) echo ' selected="selected"'; ?>>All Branches</option>
  <?php echo $selAllBranches ?>
</select> &nbsp;
<input type="submit" value="View Branch" />
</td></tr>
</table>
</form>
</td></tr>
<?php
}
?>
</table>


<?php
    chora_footer();
} else {
    fatal('404 Not Found', "$where: no such file or directory");
}
