<?php
/**
 * $Horde: chora/diff.php,v 1.43.2.13 2003/01/03 12:48:48 jan Exp $
 *
 * Copyright 2000-2003 Anil Madhavapeddy <anil@recoil.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

require_once 'lib/base.php';

hasAccess($p_user->getUserId(), "/cvs/", "r", $group_id);

/* Spawn the repository and file objects */
$fl = &CVSLib_File::getFileObject($CVS, $CVS->cvsRoot() . '/' . $where);

/* Initialise the form variables correctly.
 * If r1/r2 are empty, then use the corresponding text field instead */
if (!$r1) $r1 = $tr1;
if (!$r2) $r2 = $tr2;

/* If no context-size has been specified, default to 3. */
if (!$num) {
  $num = 3;
}

/* If no type has been specified, then default to human readable. */
if (!$ty) {
  $ty = 'h';
}

/* Unless otherwise specified, show whitespace differences. */
if (!$nows) {
  $ws = 1;
} else {
  $ws = 0;
}

/* Figure out what type of diff has been requested */
switch ($ty) {
case 'u':
    $type = CVSLIB_DIFF_UNIFIED;
    break;
case 'h':
    $type = CVSLIB_DIFF_UNIFIED;
    break;
case 's':
    $type = CVSLIB_DIFF_COLUMN;
    break;
case 'c':
    $type = CVSLIB_DIFF_CONTEXT;
    break;
case 'e':
    $type = CVSLIB_DIFF_ED;
    break;
default:
    $type = CVSLIB_DIFF_UNIFIED;
    break;
}

/* Ensure that we have valid revision numbers */
if (!CVSLib_Rev::valid($r1) || !CVSLib_Rev::valid($r2)) {
    checkError(new CVSLib_Error(CVSLIB_NOT_FOUND, 'Malformed Query'));
}

/* Cache the output of the diff for a week - it can be longer, since
 * it should never change */
header('Cache-Control: max-age=604800');

/* Title to use for html output pages */
$title = "Diff for $where between version $r1 and $r2";

/* All is ok, proceed with the diff */
switch ($type) {
case CVSLIB_DIFF_COLUMN:
    header('Content-Type: text/plain');
    echo implode("\n", CVSLib_Diff::get($CVS, $fl, $r1, $r2,
				      $type, $num, $ws));
    break;

case CVSLIB_DIFF_CONTEXT:
    header('Content-Type: text/plain');
    echo implode("\n", CVSLib_Diff::get($CVS, $fl, $r1, $r2, $type, $num, $ws));
    break;

case CVSLIB_DIFF_ED:
    header('Content-Type: text/plain');
    echo implode("\n", CVSLib_Diff::get($CVS, $fl, $r1, $r2, $type, $num, $ws));
    break;

case CVSLIB_DIFF_UNIFIED:
default:
    /* We'll need to know the mime type to modify diffs based on the mime
       type. */
    require_once 'horde-libs/MIME/Magic.php';
    $mime_type = MIME_Magic::extToMIME($fl->getExtension());

    if ($ty != 'h') {
        /* Not Human-Readable format. */
        header('Content-Type: text/plain');
        echo implode("\n", CVSLib_Diff::get($CVS, $fl, $r1, $r2,
                                            $type, $num, $ws));
    } else if (in_array($mime_type, array('image/gif', 'image/jpeg', 'image/png', 'image/bmp', 'image/tiff'))) {
        // The above are images that most web browsers can inline
        // We borrow a *large* part of this from the Human-Readable case
        $url1 = Chora::url('co', $where, array('r' => $r1));
        $url2 = Chora::url('co', $where, array('r' => $r2));
        $path = $fl->queryModulePath();

        // Get the log entry so we can display it
        $log = $fl->logs[$r2];
        $log_print = Chora::toHTML($log->queryLog());

        // Start the html output, include menu bar and headers
	chora_header(array('title' => $title, 'r1' => $r1, 'r2' => $r2, 'nows' => $nows, 'subtab' => 'column'));
        // Create a table for the two revisions, display log, and
        // print a labeled bar for the revisions.

	echo "<br />\n";
	echo "<b>Last Log Message for rev $r2:</b>\n";
	echo "<p>$log_print</p>\n";
	echo "<br />\n";
	$diff_header = array();

	array_push($diff_header, array('title' => "version $r1",
				       'link' => Chora::url('co', $where, array('r' => $r1))));


	array_push($diff_header, array('title' => "version $r2",
				       'link' => Chora::url('co', $where, array('r' => $r2))));

	echo $HTML->build_list_table_top($diff_header, 0);

        echo "  <td width=\"50%\" bgcolor=\"$HTML->COLOR_LTBACK1\"><img src=\"$url1\"></td>\n";
        echo "  <td width=\"50%\" bgcolor=\"$HTML->COLOR_LTBACK2\"><img src=\"$url2\"></td>\n";
        echo "</tr>\n";
	echo "</table>";
	chora_footer();
    } else {
        /* Human-Readable diff. */

        /* Output standard header information for the page. */
        $filename = preg_replace('/^.*\//', '', $where);
        $pathname = preg_replace('/[^\/]*$/', '', $where);

        $log = $fl->logs[$r2];
        $log_print = Chora::toHTML($log->queryLog());

	if ($num > 3) {
	  $subtab = "long";
	} else {
	  $subtab = "diff";
	}
	chora_header(array('title' => $title, 'r1' => $r1, 'r2' => $r2, 'nows' => $nows, 'subtab' => $subtab));
	if ($nows) {
	  $extraLink='<input type="checkbox" checked="checked" onClick="document.location=\'' . Chora::url('diff', $where, array('nows' => 0, 'ty' => $ty, 'r1' => $r1, 'r2' => $r2, 'num' => $num)) . "'\"> Ignore Whitespace";
	} else {
	  $extraLink='<input type="checkbox" onClick="document.location=\'' . Chora::url('diff', $where, array('nows' => 1, 'ty' => $ty, 'r1' => $r1, 'r2' => $r2, 'num' => $num)) . "'\"> Ignore Whitespace";
	}
	echo '<div align="left"><p>' . $extraLink . "</p></div>\n";

	echo "<br />\n";
	echo "<b>Last Log Message for rev $r2:</b>\n";
	echo "<p>$log_print</p>\n";
	echo "<br />\n";

	$diff_header = array();

	array_push($diff_header, array('title' => "version $r1",
				       'link' => Chora::url('co', $where, array('r' => $r1))));


	array_push($diff_header, array('title' => "version $r2",
				       'link' => Chora::url('co', $where, array('r' => $r2))));

	echo $HTML->build_list_table_top($diff_header, 0);


        /* Retrieve the tree of changes from CVSLib */
        $lns = CVSLib_Diff::humanReadable(
            CVSLib_Diff::get($CVS, $fl, $r1, $r2, CVSLIB_DIFF_UNIFIED,
                $num, $ws));
        /* TODO: check for errors here (CVSLib_Error returned) - avsm */
        /* Is the diff empty? */
        if (!sizeof($lns)) {
?>
<tr>
  <td colspan="2">&nbsp;</td>
</tr>
<tr>
  <td colspan="2" align="center">
    <b>- No Viewable Change -</b>
  </td>
</tr>
<?php
        } else {
            /* Iterate through every header block of changes */
            foreach ($lns as $header) {
                $lefthead = Text::htmlspaces(@$header['oldline']);
                $righthead = Text::htmlspaces(@$header['newline']);
                $headfunc = Text::htmlspaces(@$header['function']);
		echo '<tr bgcolor="' . $HTML->LISTITEM_SELECTED_COLOR . "\">\n";
		echo " <td width=\"50%\">";
		echo "   &nbsp; <b>Line $lefthead </b> $headfunc </td>\n";
		echo " <td width=\"50%\">";
		echo "   &nbsp; <b>Line $righthead</b> $headfunc </td>\n";
		echo "</tr>\n";

                /* Each header block consists of a number of changes
                   (add, remove, change) */
                foreach ($header['contents'] as $change) {
                    switch ($change['type']) {
                    case CVSLIB_DIFF_ADD:
                        foreach ($change['lines'] as $line) {
                            $line = Text::htmlspaces($line);
?>
<tr>
 <td class="hr-diff-noadd">&nbsp;</td>
 <td class="hr-diff-add"><tt><?php echo !empty($line) ? $line : '&nbsp;' ?></tt></td>
</tr>
<?php

                        }
                        break;
                    case CVSLIB_DIFF_REMOVE:
                        foreach ($change['lines'] as $line) {
                            $line = Text::htmlspaces($line);
?>
<tr>
  <td class="hr-diff-remove">
    <tt><?php echo $line ?></tt>
  </td>
  <td class="hr-diff-noremove">&nbsp;</td>
</tr>
<?php
                        }
                        break;
                    case CVSLIB_DIFF_EMPTY:
                        $line = Text::htmlspaces($change['line']);
?>

<tr>
 <td bgcolor="<?php echo $HTML->COLOR_LTBACK2; ?>"><tt>&nbsp;<?php echo $line ?></tt></td>
 <td bgcolor="<?php echo $HTML->COLOR_LTBACK2; ?>"><tt>&nbsp;<?php echo $line ?></tt></td>
</tr>

<?php
                        break;
                    case CVSLIB_DIFF_CHANGE:
		      $oldsize = sizeof($change['old']);
		      $newsize = sizeof($change['new']);

		      for ($i = 0; $i < max($oldsize, $newsize); $i++) {
			$left = Text::htmlspaces($change['old'][$i]);
			$right = Text::htmlspaces($change['new'][$i]);

			echo "<tr>\n";
			if ($left) {
			  echo "  <td class=\"hr-diff-change\">\n";
			  echo "     <tt>$left</tt>\n";
			  echo "  </td>\n";
			} else if ($i < $oldsize) {
			  echo "  <td class=\"hr-diff-change\"> &nbsp; </td>\n";
			} else {
			  echo "  <td class=\"hr-diff-nochange\"> &nbsp; </td>\n";
			}
			
			if ($right) {
			  echo "  <td class=\"hr-diff-change\">\n";
			  echo "     <tt>$right</tt>\n";
			  echo "  </td>\n";
			} else if ($i < $newsize) {
			  echo "  <td class=\"hr-diff-change\"> &nbsp; </td>\n";
			} else {
			  echo "  <td class=\"hr-diff-nochange\"> &nbsp; </td>\n";
			}
			echo "</tr>\n";

		      }
		      
                        break;
                    }
                }
            }
        }

        /* print legend */
?>
</table>

    <p> &nbsp; </p>

<table width="30%">
<tr><td>
<?php
    $HTML->box1_top("Legend", 1, '', 0);
?>
     <tr>
      <td width="50%" align="center" class="hr-diff-remove"><?php echo "Removed in v. $r1"; ?></td>
      <td width="50%" class="hr-diff-noremove">&nbsp;</td>
     </tr>
     <tr class="hr-diff-change">
      <td align="center" colspan="2">changed lines</td>
     </tr>
     <tr>
      <td class="hr-diff-noadd">&nbsp;</td>
      <td align="center" class="hr-diff-add"><?php echo "Added in v. $r2"; ?>
<?php

	$HTML->box1_bottom();
?>
</td></tr>
</table>

<?php
	chora_footer();
    }
    break;
}

?>
