<?php
// $Horde: chora/lib/Chora.php,v 1.15.2.11 2002/12/22 13:34:53 jan Exp $

require_once 'lib/constants.php';
require_once 'lib/CVSLib.php';

/* Variable we wish to propagate across web pages
 *  sbt = Sort By Type (name, age, author, etc)
 *  ha  = Hide Attic Files
 *  ord = Sort order
 *
 * Obviously, defaults go into $defaultActs :)
 * TODO: defaults of 1 will not get propagated correctly - avsm
 * XXX: Rewrite this propagation code, since it sucks - avsm
 */

$defaultActs = array('sbt'   => $conf['options']['defaultsort'],
                     'sa'    => 0,
                     'login' => 0,
                     'ord'   => CVSLIB_SORT_ASCENDING,
		     'group_id' => 0);

/* See if any have been passed as GET variables, and if
 * so, assign them into the acts array */
while (list($key,) = each($defaultActs)) {
  if ($$key) {
    $acts[$key] = $$key;
  } else {
    $acts[$key] = $defaultActs[$key];
  }
}

$CVS = new CVSLib();

/**
 * Output an error page with relevant HTTP error headers
 *
 * @param errcode The HTTP error number and text
 * @param errmsg The verbose error message to be displayed
 */
function fatal($errcode, $errmsg)
{
  exit_error("Chora Error $errcode", $errmsg);
}

/**
 * Given a return object from a CVSLib call, make sure
 * that it's not a CVSLib_Error object.
 * @param e Return object from a CVSLib call
 */
function checkError($e)
{
    if (is_object($e) && ($e->id() == CVSLIB_ERROR)) {
        fatal($e->error_header(), $e->error_body());
    }
}

$where = $f;

/* Override $where with PATH_INFO if appropriate */
if ($conf['options']['use_path_info'] && isset($_SERVER['PATH_INFO'])) {
    $where = $_SERVER['PATH_INFO'];
}

/* Location relative to the CVSROOT */
$where = stripslashes($where);
$where = preg_replace("|^/|", '', $where);
$where = preg_replace("|\.\.|", '', $where);
$where = preg_replace('|/$|', '', $where);

/* Location of this script (e.g. /chora/cvs.php) */
$scriptName = preg_replace('|^/?|', '/', $_SERVER['REQUEST_URI']);
$scriptName = preg_replace('|/$|', '', $scriptName);

if ($group_id) {
  $aGroup = &group_get_object($group_id);
  if (!$aGroup->usesCVS()) {
    fatal('300', 'Group Does not use CVS');
  }
  $conf['paths']['cvsRoot'] = '/cvsroot/' . $aGroup->getUnixName();
  $conf['options']['introTitle'] = $aGroup->getPublicName();
  $conf['options']['cvsRootName'] = $aGroup->getPublicName();
  $cvsroot = '/cvsroot/' . $aGroup->getUnixName();
} else if ($rt) {
  // do some magic to select the group by name
  $sql = "SELECT group_id FROM groups WHERE unix_group_name='$rt'";
  $res = db_query($sql);
  if ($res && db_numrows($res) > 0 && $group_id = db_result($res, 0, 'group_id')) {
    $acts['group_id'] = $group_id;
    $split = preg_split('|/|', $_SERVER['SCRIPT_NAME']);
    $script = array_pop($split);
    $script = preg_replace('|\.php$|', '', $script);
    $newUrl = Chora::url($script, $where);
    header("Location: " . $newUrl);
  } else {
    fatal(404, 'Group does not exist');
  }
} else {
  fatal(404, 'No Group Specified');
}
  

$fullname = "$cvsroot/$where";

if (!@is_dir($cvsroot)) {
    fatal("500 Internal Error", "CVSROOT not found! This could be a misconfiguration by the server administrator, or the server could be having temporary problems. Please try again later.");
}


/**
 * Chora Base Class.
 *
 * @author Anil Madhavapeddy <avsm@horde.org>
 * @version $Revision: 1.3 $
 * @package chora
 */
class Chora {

    function whereMenu()
    {
        global $where, $HTML;

        $bar = '';
        $wherePath = '';

        foreach (explode('/', $where) as $dir) {
	  // assumes that we're inside of a tab_bar already, but building it by hand here
	  $wherePath .= "/$dir";
	  echo " / ";
	  $HTML->tab_entry(Chora::url('cvs', $wherePath), '', $dir);
        }
        return $bar;
    }

    /*
     * Return an array with the names of any of the variables we
     * need to keep, that are different from the defaults
     *
     * @ret Array containing names/vals of differing variables
     */
    function differingVars()
    {
        global $acts, $defaultActs;
        reset($acts);
        $ret = array();
        while (list($key, $val) = each($acts)) {
            if ($val != $defaultActs[$key]) {
                $ret[$key] = $val;
            }
        }
        return $ret;
    }

    /**
     * Generate a series of HIDDEN input forms based on the
     * GET parameters which are different from the defaults
     *
     * @param except Array of exceptions to never output
     * @return A set of INPUT tags with the different variables
     */
    function generateHiddens($except = array())
    {
        global $acts;
        $toOut = Chora::differingVars();
        //$ret = Horde::formInput() . "\n";
	$ret = "";
        while (list($key, $val) = each($toOut)) {
            if (is_array($except) && !in_array($key, $except)) {
                $ret .= "<input type=\"hidden\" name=\"$key\" value=\"$val\" />\n";
            }
        }
        return $ret;
    }

    /**
     * Convert a commit-name into whatever the user wants
     * @param commit name
     * @return transformed name
     */
    function showAuthorName($name, $fullname = false)
    {

      if (user_is_valid($name)) {
	return "<a href=\"/users/$name\">$name</a>\n";
      } else {
	return "$name\n";
      }
    }

    /**
     * Generate a URL that links into Chora.
     * @param script Name of the Chora script to link into
     * @param uri Any PATH_INFO portion that should be included
     * @param args Key/value pair of any GET parameters to append
     * @param anchor Anchor entity name
     */
    function url($script, $uri = '', $args = array(), $anchor = '')
    {
      global $conf;

	$url = "/cvs/$script.php";
        $uri = rawurlencode($uri);

        $arglist = array_merge(Chora::differingVars(), $args);
	if ($conf['options']['use_path_info']) {
	  $url .= '/' . $uri;
	} else {
	  $arglist['f'] = $uri;
	}

        if (!isset($_COOKIE[session_name()])) {
            $arglist[urlencode(session_name())] = session_id();
        }

        $argarr = array();
        foreach ($arglist as $key => $val) {
            if (!empty($val)) {
                $val = htmlspecialchars($val);
                $argarr[] = "$key=$val";
            }
        }

        if (sizeof($argarr) > 0) {
            $url = "$url?" . implode('&', $argarr);
            $glue = '&';
        } else {
            $glue = '?';
        }

        if (!empty($anchor)) {
            $url .= "#$anchor";
        }

        $url = preg_replace('|/\?|', '?', $url);
        $url = preg_replace('|%2F|', '/', $url);
        $url = preg_replace('|/+|', '/', $url);
        $url = str_replace('&', '&amp;', $url);

        return $url;
    }

    /**
     * Turn text into HTML.
     *
     * @access public
     *
     * @param string $input  An url-decoded string, \n-separated for lines.
     *
     * @return string  The converted HTML.
     */
    function toHTML($text)
    {

      // co-opted by SF utils
      return util_make_links(htmlentities($text));
    }

    /**
     * Check if the given item is restricted from being shown.
     * @return boolean whether or not the item is allowed to be displayed
     **/
    function isRestricted($item)
    {
      /** commented out until we've got a good way to do this **
        global $conf, $cvsroots, $cvsroot;
        static $restricted;

        if (!isset($restricted)) {
            $restricted = array();
            if (isset($conf['restrictions']) && is_array($conf['restrictions'])) {
                $restricted = $conf['restrictions'];
            }

            foreach ($cvsroots as $key => $val) {
                if ($cvsroot == $val['location']) {
                    if (isset($val['restrictions']) && is_array($val['restrictions'])) {
                        $restricted = array_merge($restricted, $val['restrictions']);
                        break;
                    }
                }
            }
        }

        if (!empty($restricted) && is_array($restricted) && count($restricted)) {
            for ($i = 0; $i < count($restricted); $i++) {
                if (preg_match('|' . str_replace('|', '\|', $restricted[$i]) . '|', $item)) {
                    return true;
                }
            }
        }


	** **/
        return false;
    }

}
?>
